# -*- coding: utf-8 -*-
"""Integration tests for the I18NDict."""

# python imports
try:
    import unittest2 as unittest
except ImportError:
    import unittest

# zope imports
from zope.component import provideUtility
from zope.interface import implementer

# local imports
from ps.zope.i18nfield import (
    interfaces,
    storage,
    utils,
)


@implementer(interfaces.ILanguageAvailability)
class LanguageAvailability(object):

    def getDefaultLanguage(self, combined=False):
        return 'en'

    def getAvailableLanguages(self, combined=False):
        return ['de', 'en', 'es', 'fr', 'pt']

    def getLanguages(self, combined=False):
        pass

    def getLanguageListing(self, combined=False):
        pass


class TestI18NDict(unittest.TestCase):
    """Test case for the I18n language dictionary."""

    def setUp(self):
        super(TestI18NDict, self).setUp()
        provideUtility(LanguageAvailability())
        self._old_func = utils.get_language

    def tearDown(self):
        super(TestI18NDict, self).tearDown()
        utils.get_language = self._old_func

    def test_standard(self):
        """Test that the dictionary access with no default language."""
        data = storage.I18NDict()
        self.assertIsNone(data.get('en'))
        data['en'] = 'English'
        self.assertEqual(data.get('en'), 'English')
        data['es'] = 'Español'
        self.assertEqual(data.get('es'), 'Español')
        self.assertEqual(len(list(data.keys())), 2)

    def test_default(self):
        """Test the default value method with a default language set."""
        data = storage.I18NDict()
        data.default_language = 'es'

        # no values - fallback is None
        self.assertIsNone(data.get_default_value())

        # one value - fallback is the first value found
        data['fr'] = 'French'
        self.assertEqual(data.get_default_value(), 'French')

        # two values - fallback comes from global default (ILanguageSettings)
        data['en'] = 'English'
        self.assertEqual(data.get_default_value(), 'English')

        # three values - fallback comes from I18nDict default language
        data['es'] = 'Español'
        self.assertEqual(data.get_default_value(), 'Español')

        # set to non-existent default language - default from ILanguageSettings
        data.default_language = 'de'
        self.assertEqual(data.get_default_value(), 'English')

        # default comes from global registry
        # self.assertEqual(data.get_default_value(), u'French')

        # default comes from I18nDict internally saved default language
        data.default_language = 'en'
        self.assertEqual(data.get_default_value(), 'English')

    def test_no_default(self):
        """Test the default value method with no default langauge set."""
        data = storage.I18NDict()

        # no values - default is None
        self.assertIsNone(data.get_default_value())

        # one value - fallback is first value found
        data['fr'] = 'French'
        self.assertEqual(data.get_default_value(), 'French')

        # two values - global registry default is used
        data['en'] = 'English'
        self.assertEqual(data.get_default_value(), 'English')

        # three values - global registry default is still used
        data['es'] = 'Español'
        self.assertEqual(data.get_default_value(), 'English')

    def test_unicode_representation(self):
        """Test the __unicode__ method of the dictionary."""
        data = storage.I18NDict()
        data.required = True

        # no request, no values
        self.assertEqual(str(data), '')

        # no request - fallback to first value
        data['fr'] = 'French'
        self.assertEqual(str(data), 'French')

        # no request - fallback to global default (English)
        data['en'] = 'English'
        data['es'] = 'Español'
        self.assertEqual(str(data), 'English')

        # request language Spanish
        utils.get_language = lambda: 'es'
        self.assertEqual(str(data), 'Español')

        # request language French
        utils.get_language = lambda: 'fr'
        self.assertEqual(str(data), 'French')

        # request language English
        utils.get_language = lambda: 'en'
        self.assertEqual(str(data), 'English')

    def test_string_representation(self):
        """Test the __str__ method of the dictionary."""
        data = storage.I18NDict()
        data.default_language = 'es'
        data.required = True

        # no request, no values
        self.assertEqual(str(data), '')

        # no request - fallback to first value
        data['fr'] = 'French'
        self.assertEqual(str(data), 'French')

        # no value for request language - fallback to internal I18nDict default
        data['en'] = 'English'
        data['es'] = 'Español'
        utils.get_language = lambda: 'pt'
        self.assertEqual(str(data), 'Español'.encode('utf-8'))

        # request language Spanish
        utils.get_language = lambda: 'es'
        self.assertEqual(str(data), 'Español'.encode('utf-8'))

        # request language French
        utils.get_language = lambda: 'fr'
        self.assertEqual(str(data), 'French')

        # request language English
        utils.get_language = lambda: 'en'
        self.assertEqual(str(data), 'English')

    def test_not_required(self):
        """Test the __unicode__ method without a required field."""
        data = storage.I18NDict()
        data.default_language = 'es'
        data.required = False
        data['fr'] = 'French'
        data['de'] = 'German'
        data['es'] = 'Español'

        # no request, no value for default language 'en'
        self.assertEqual(str(data), '')

        # with required
        data.required = True
        self.assertEqual(str(data), 'Español')

    def test_from_text(self):
        """Test the from_text method."""
        data = storage.I18NDict.from_text('Test value')
        self.assertEqual(data, {'__default_value': 'Test value'})
        self.assertEqual(str(data), 'Test value')

    def test_to_dict(self):
        original = storage.I18NDict()
        original['en'] = 'English'
        original['es'] = 'Español'
        self.assertEqual(len(original), 2)
        copy = original.to_dict()
        self.assertEqual(len(copy), 2)
        self.assertIn('en', list(copy.keys()))
        self.assertIn('Español', list(copy.values()))

    def test_copy(self):
        original = storage.I18NDict()
        original.default_language = 'es'
        original.required = True
        original['en'] = 'English'
        original['es'] = 'Español'
        self.assertEqual(len(original), 2)
        copy = original.copy()
        self.assertEqual(len(copy), 2)
        self.assertIn('es', list(copy.keys()))
        self.assertIn('English', list(copy.values()))
        self.assertEqual(copy.default_language, original.default_language)
        self.assertEqual(copy.required, original.required)

    def test_set_item(self):
        """Test the checks from __setitem__ method."""
        data = storage.I18NDict()
        data['fr'] = 'French'
        self.assertIn('fr', data)
        data['__default_value'] = 'Default value'
        self.assertIn('Default value', list(data.values()))
        data['non-existent'] = 'Invalid'
        self.assertEqual(len(data), 2)
        data['en'] = ''
        self.assertEqual(len(data), 2)
        self.assertNotIn('en', data)

    def test_update(self):
        """Test the update method."""
        data = storage.I18NDict({'en': 'English', 'invalid': 'blah'})
        self.assertEqual(len(data), 1)
        self.assertIn('en', data)

        data.update({
            'es': 'Español',
            'fr': 'François',
            'fake': 'Non-value',
        })
        self.assertEqual(len(data), 3)
        self.assertIn('Español', list(data.values()))
        self.assertIn('François', list(data.values()))

        data.update([
            ('de', 'Deutsch'),
            ('pt', 'Português'),
            ('slang', 'Jibber jabber'),
        ])
        self.assertEqual(len(data), 5)
        self.assertIn('de', list(data.keys()))
        self.assertIn('pt', list(data.keys()))

    def test_add(self):
        """Test the add method."""
        data = storage.I18NDict()
        data.add('de', 'Deutsch')
        self.assertEqual(len(data), 1)
        self.assertIn('de', data)
        data.add('fake', 'Fake language')
        self.assertEqual(len(data), 1)

    def test_remove(self):
        """Test the remove method."""
        data = storage.I18NDict({'en': 'English', 'de': 'Deutsch'})
        self.assertEqual(len(data), 2)
        data.remove('de')
        self.assertEqual(len(data), 1)
        self.assertNotIn('de', data)
        data.remove('not')
        self.assertEqual(len(data), 1)

    def test_to_text(self):
        """Test the to_text method."""
        data = storage.I18NDict({'en': 'English', 'de': 'Deutsch'})
        self.assertIn('English', data.to_text())
        self.assertIn('Deutsch', data.to_text())
        data['es'] = 'El texto en español'
        self.assertIn('El texto en español', data.to_text())

    def test_nonzero_not_required(self):
        """Test the __nonzero__ method."""
        data = storage.I18NDict()
        data.required = False

        # no request, no values
        self.assertFalse(bool(data))

        # no request - no value for default language
        data['fr'] = 'Français'
        self.assertFalse(bool(data))

        # no request - no value for default language
        data['es'] = 'Español'
        self.assertFalse(bool(data))

        # no request - with a value for default language
        data['en'] = 'English'
        self.assertTrue(bool(data))

    def test_nonzero_required(self):
        """Test the __nonzero__ method."""
        data = storage.I18NDict()
        data.required = True

        # no request, no values
        self.assertFalse(bool(data))

        # no request - fallback to first value found
        data['fr'] = 'Français'
        self.assertTrue(bool(data))

        # no request - fallback to first value found
        data['es'] = 'Español'
        self.assertTrue(bool(data))

        # no request - with a value for default language
        data['en'] = 'English'
        self.assertTrue(bool(data))
